#include "CNCFile.h"
#include "TextEditView.h"
#include <wx/wfstream.h>

using namespace std;

void CNCFile::ReadHead(int size, const char * d)
{
	headsize=size;
	for(int i = 0; i < size; i++)
		head[i]=d[i];
	GetMainFrame()->RefreshOpenFileList();
	TextEditView *view = (TextEditView *)GetFirstView();
	if (view)
		view->RefreshTitle();
}

CNCFile::CNCFile()
{
	headsize=0;
	ActualizeToolbar();
	GetMainFrame()->RefreshOpenFileList();
}

CNCFile::~CNCFile()
{
	ActualizeToolbar();
}

void CNCFile::SetFileEnde(char * bin, int len)
{
	int x;
	for(x=0; x < len; x++)
		if(bin[x] == 0x3a && ( bin[x+1] == 0x1a || bin[x+1] == 0x03 || bin[x+1] == 0x01 ))
		{
			if(bin[x-1]!='\n')
			{
				bin[x]='\n';
				bin[x+1]=0;
			}
			else
			{
				bin[x]=0;
			}
		}
}

void CNCFile::WriteHead(FILE * fp)
{
	for(int i = 0; i < headsize; i++)
		fputc(head[i], fp);
}

void CNCFile::WriteFooter(FILE * fp)
{
	int x;
	fputc(0x3a, fp);
	fputc(0x1a, fp);
	x=ftell(fp);
	x=128-(x%128);
	for(;x > 0;x--)
		fputc(0, fp);
}

void CNCFile::Modify(bool mod)
{
	//Satus nicht neu setzten
	if(IsModified() == mod)
		return;

	wxDocument::Modify(mod);
	ActualizeToolbar();
	TextEditView *view = (TextEditView *)GetFirstView();
	if (view)
		view->RefreshTitle();
}


void CNCFile::copy()
{
	cout << "CNCFile::copy()\n";
	TextEditView *view = (TextEditView *)GetFirstView();
	view->textsw->Copy();
}

void CNCFile::paste()
{
	cout << "CNCFile::paste()\n";
	TextEditView *view = (TextEditView *)GetFirstView();
	view->textsw->Paste();
}

void CNCFile::cut()
{
	cout << "CNCFile::cut()\n";
	TextEditView *view = (TextEditView *)GetFirstView();
	view->textsw->Cut();
}

void CNCFile::undo()
{
	cout << "CNCFile::undo()\n";
	TextEditView *view = (TextEditView *)GetFirstView();
	if(view)
		view->textsw->Undo();
}

void CNCFile::redo()
{
	cout << "CNCFile::redo()\n";
	TextEditView *view = (TextEditView *)GetFirstView();
	if(view)
		view->textsw->Redo();
}

void CNCFile::search()
{
	//TODO:Search
	cout << "CNCFile::search()=Not yet implemeted\n";
}

void CNCFile::save_as_pdf()
{
	//TODO:PDF
	cout << "CNCFile::save_as_pdf()=Not yet implemeted\n";
}

bool CNCFile::OnOpenDocument(const wxString& filename)
{
	wxFFileInputStream in(filename, "rb");
	if(!in.IsOk())
	{
		wxMessageBox("Datei konnte nicht geoefnet werden!", "Fehler", wxOK | wxICON_ERROR);
		return false;
	}

	GetMainFrame()->ActivateNewstFile();

	return OpenFile(in, filename);
}

void CNCFile::SelectAll()
{
	TextEditView *view = (TextEditView *)GetFirstView();
	if(view)
		view->textsw->SelectAll();
}

int CNCFile::GetActiveToolbars()
{
	int tools = ID_TB_SAVE_AS_PDF | ID_TB_SETTINGS | ID_TB_CUT
		| ID_TB_COPY | ID_TB_SEARCH;
	TextEditView *view = (TextEditView *)GetFirstView();
	if(view)
	{
		if(view->textsw->CanUndo())
			tools|=ID_TB_UNDO;

		if(view->textsw->CanRedo())
			tools|=ID_TB_REDO;

		if(view->textsw->CanPaste())
			tools|=ID_TB_PASTE;
	}

	return tools;
}

void CNCFile::OnFileOpened()
{
	Modify(false);
	UpdateAllViews();
	SetDocumentSaved();

	TextEditView *view = (TextEditView *)GetFirstView();
	if(view)
	{
		view->textsw->EmptyUndoBuffer();
		view->textsw->SetSavePoint();
	}
	ActualizeToolbar();
}
