#include "MyDocManager.h"
#include "BinInputStream.h"
#include "CNCFile.h"
#include "TextEditView.h"
#include "TextWindow.h"

using namespace std;


// Try to get a more suitable parent frame than the top window,
// for selection dialogs. Otherwise you may get an unexpected
// window being activated when a dialog is shown.
static wxWindow* wxFindSuitableParent()
{
    wxWindow* parent = wxTheApp->GetTopWindow();

    wxWindow* focusWindow = wxWindow::FindFocus();
    if (focusWindow)
    {
        while (focusWindow &&
                !focusWindow->IsKindOf(CLASSINFO(wxDialog)) &&
                !focusWindow->IsKindOf(CLASSINFO(wxFrame)))

            focusWindow = focusWindow->GetParent();

        if (focusWindow)
            parent = focusWindow;
    }
    return parent;
}

wxDocument * MyDocManager::NewDocument(const wxString & extension, long flags)
{
    wxDocument* docToClose = NULL;

    // If we've reached the max number of docs, close the
    // first one.
    if ( (int)GetDocuments().GetCount() >= m_maxDocsOpen )
    {
        wxDocument *doc = (wxDocument *)GetDocuments().GetFirst()->GetData();
        docToClose = doc;
    }

	wxDocTemplate *temp = FindTemplateForPath(extension);

	if (temp)
	{
		if (docToClose)
		{
			if (!CloseDocument(docToClose, false))
			{
				return NULL;
			}
		}

		wxDocument *newDoc = temp->CreateDocument(wxEmptyString, flags);

		if (newDoc)
		{
			newDoc->SetDocumentName(temp->GetDocumentName());
			newDoc->SetDocumentTemplate(temp);
			if (!newDoc->OnNewDocument() )
			{
				 // Document is implicitly deleted by DeleteAllViews
				 newDoc->DeleteAllViews();
				 return NULL;
			}
		}
		return newDoc;
	}
	else
		return (wxDocument *) NULL;
}

wxDocument * MyDocManager::Open(PrjFile * file, int prjId)
{
	std::cout << "Open called\n";

	BinInputStream in(file->GetData(), file->GetSize());

	wxString filename;
	filename << "prj://" << prjId << "/" << file->GetName() << "." << ((file->GetTyp()==1) ? "prog" : "uprog");

	//FindTemplateForPath(path);
	//TODO:Implement Open

	CNCFile * rfile;

	rfile=((CNCFile *)NewDocument(((file->GetTyp()==1) ? "*.prog" : "*.uprog"), 0));
	rfile->OpenFile(in, filename);

	return rfile;
}

// Prompts user to open a file, using file specs in templates.
// Must extend the file selector dialog or implement own; OR
// match the extension to the template extension.

wxDocTemplate *MyDocManager::SelectDocumentPath(wxDocTemplate **templates,
#if defined(__WXMSW__) || defined(__WXGTK__) || defined(__WXMAC__)
                                                int noTemplates,
#else
                                                int WXUNUSED(noTemplates),
#endif
                                                wxString& path,
                                                long WXUNUSED(flags),
                                                bool WXUNUSED(save))
{
    // We can only have multiple filters in Windows and GTK
#if defined(__WXMSW__) || defined(__WXGTK__) || defined(__WXMAC__)
    wxString descrBuf;

	wxString allExtension;

    int i;
    for (i = 0; i < noTemplates; i++)
    {
        if (templates[i]->IsVisible())
        {
            // add a '|' to separate this filter from the previous one
            if ( !descrBuf.empty() )
                descrBuf << wxT('|');

            descrBuf << templates[i]->GetDescription()
                << wxT(" (") << templates[i]->GetFileFilter() << wxT(") |")
                << templates[i]->GetFileFilter();
			allExtension << templates[i]->GetFileFilter() << ";";
        }
    }

	allExtension << "|" << descrBuf;
	descrBuf="";
    descrBuf << "Alle bekennaten Dateien|" << allExtension;

#else
    wxString descrBuf = wxT("*.*");
#endif

    int FilterIndex = -1;

    wxWindow* parent = wxFindSuitableParent();

    wxString pathTmp = wxFileSelectorEx(_("Select a file"),
                                        m_lastDirectory,
                                        wxEmptyString,
                                        &FilterIndex,
                                        descrBuf,
                                        0,
                                        parent);

    wxDocTemplate *theTemplate = (wxDocTemplate *)NULL;
    if (!pathTmp.empty())
    {
        if (!wxFileExists(pathTmp))
        {
            wxString msgTitle;
            if (!wxTheApp->GetAppName().empty())
                msgTitle = wxTheApp->GetAppName();
            else
                msgTitle = wxString(_("File error"));

            (void)wxMessageBox(_("Sorry, could not open this file."), msgTitle, wxOK | wxICON_EXCLAMATION,
                parent);

            path = wxEmptyString;
            return (wxDocTemplate *) NULL;
        }
        m_lastDirectory = wxPathOnly(pathTmp);

        path = pathTmp;

        // first choose the template using the extension, if this fails (i.e.
        // wxFileSelectorEx() didn't fill it), then use the path
        if ( FilterIndex != -1 && FilterIndex != 0)
            theTemplate = templates[FilterIndex-1];
        if ( !theTemplate )
            theTemplate = FindTemplateForPath(path);
        if ( !theTemplate )
        {
            // Since we do not add files with non-default extensions to the FileHistory this
            // can only happen if the application changes the allowed templates in runtime.
            (void)wxMessageBox(_("Sorry, the format for this file is unknown."),
                                _("Open File"),
                                wxOK | wxICON_EXCLAMATION, wxFindSuitableParent());
        }
    }
    else
    {
        path = wxEmptyString;
    }

    return theTemplate;
}

void MyDocManager::SetShowControlCharacter(bool cc)
{
	wxDocument * doc;

	m_control_character = cc;
	for(unsigned int i = 0; i < GetDocuments().GetCount(); i++)
	{
		doc = (wxDocument *)GetDocuments().Item(i)->GetData();
		if(doc->GetDocumentName()=="Programm" || doc->GetDocumentName()=="Unterprogramm")
		{
			if(doc->GetFirstView())
			{
				((TextEditView *)doc->GetFirstView())->textsw->SetViewWhiteSpace(cc);
			}
		}
	}
}
















