#include "MyFrame.h"
#include "lang.h"
#include <wx/docview.h>
#include "TextWindow.h"
#include "CNCFile.h"
#include "TextEditView.h"
#include <wx/treebook.h>
#include "Images.h"
#include <wx/aboutdlg.h>
#include "icon.xpm"
#include "ProjectView.h"
#include <wx/sizer.h>


using namespace std;

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
    EVT_ERASE_BACKGROUND(MyFrame::OnEraseBackground)
    EVT_SIZE(MyFrame::OnSize)
    EVT_MENU(wxID_EXIT, MyFrame::OnExit)
    EVT_MENU(wxID_ABOUT, MyFrame::OnAbout)
    EVT_AUI_PANE_CLOSE(MyFrame::OnPaneClose)
    EVT_AUINOTEBOOK_ALLOW_DND(wxID_ANY, MyFrame::OnAllowNotebookDnD)
    EVT_AUINOTEBOOK_PAGE_CLOSE(wxID_ANY, MyFrame::OnNotebookPageClose)
    EVT_AUINOTEBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnNotebookChanged)
    //EVT_TEXT(ID_TextEditWindow, MyFrame::OnDocChanged)

    EVT_MENU(wxID_COPY, MyFrame::copy)
    EVT_MENU(wxID_PASTE, MyFrame::paste)
    EVT_MENU(wxID_CUT, MyFrame::cut)
    EVT_MENU(ID_UNDO, MyFrame::undo)
    EVT_MENU(ID_REDO, MyFrame::redo)
    EVT_MENU(wxID_FIND, MyFrame::search)
    EVT_MENU(ID_FIND_CONTINUE, MyFrame::search)
    EVT_MENU(wxID_REPLACE, MyFrame::replace)
    EVT_MENU(wxID_SELECTALL, MyFrame::SelectAll)
    //EVT_MENU(wxID_ABOUT, MyFrame::save_as_pdf)


    EVT_MENU(ID_PSHOW_PRJ, MyFrame::SetView)
    EVT_MENU(ID_PSHOW_LOG, MyFrame::SetView)
    EVT_MENU(ID_PSHOW_SETTINGS, MyFrame::SetView)
    EVT_MENU(ID_SHOWTB_FILE, MyFrame::SetView)
    EVT_MENU(ID_SHOWTB_EDIT, MyFrame::SetView)
    EVT_MENU(ID_SHOWTB_FLOPPY, MyFrame::SetView)
    EVT_MENU(ID_SHOWTB_VIEW, MyFrame::SetView)
    EVT_MENU(ID_VIEW_RESET, MyFrame::LoadDefaultView)


    EVT_MENU(ID_TB_VIEW_CC, MyFrame::toggleControlCharacter)
    EVT_MENU(ID_VIEW_CC, MyFrame::toggleControlCharacter)


    EVT_MENU(ID_NEW_PROGRAMM, MyFrame::NewProgramm)
    EVT_MENU(ID_NEW_UPROGRAMM, MyFrame::NewUProgramm)
    EVT_MENU(ID_NEW_PROJECT, MyFrame::NewProject)

    //Drag & Drop Tree
    EVT_TREE_BEGIN_DRAG(ID_Tree, MyFrame::TreeBeginDrag)

	//Neu Menu initialisieren
    EVT_BUTTON(ID_NEW_TOOLBAR_DROPDOWN, MyFrame::FillNewPulldown)
    EVT_MENU(ID_NEW_TOOLBAR, MyFrame::NewToolbar)

	EVT_TREE_ITEM_ACTIVATED(ID_Tree, MyFrame::TreeItemActivated)

//	EVT_PG_CHANGED(wxT("Kommentar"), MyFrame::OnCommentChanged)
	EVT_PG_CHANGED(wxID_ANY, MyFrame::OnCommentChanged)

	EVT_SCI_CHANGE(wxID_ANY, MyFrame::OnDocChanged)

    EVT_FIND(wxID_ANY, MyFrame::OnFindDialog)
    EVT_FIND_NEXT(wxID_ANY, MyFrame::OnFindDialog)
    EVT_FIND_REPLACE(wxID_ANY, MyFrame::OnFindDialog)
    EVT_FIND_REPLACE_ALL(wxID_ANY, MyFrame::OnFindDialog)
    EVT_FIND_CLOSE(wxID_ANY, MyFrame::OnFindDialog)


END_EVENT_TABLE()


MyFrame::MyFrame(MyDocManager *manager,
		wxFrame *frame,
		wxWindowID id,
		const wxString& title,
		const wxPoint& pos,
		const wxSize& size,
		const long type):
		wxDocParentFrame(manager, frame, id, title, pos, size, type)
{
    // tell wxAuiManager to manage this frame
    m_mgr.SetManagedWindow(this);

	m_manager=manager;

	m_config = 	new wxConfig("CNiFloppy");
	//Imageliste initialisieren
	Image::Init();

    // set frame icon
#ifdef __WXMSW__
    SetIcon(wxIcon(_T("CNIFLOPPYICO")));
#else
    SetIcon(wxIcon(icon_xpm));
#endif

    // set up default notebook style
    m_notebook_style = wxAUI_NB_DEFAULT_STYLE | wxAUI_NB_TAB_EXTERNAL_MOVE | wxNO_BORDER | wxAUI_NB_WINDOWLIST_BUTTON;
    m_notebook_theme = 0;

   // create menu
	CreateMenuBar();

	//Zuletzt verwendete Dateien
	m_manager->FileHistoryUseMenu(m_historyMenu);
	m_manager->FileHistoryLoad(*m_config);

    CreateStatusBar();
    GetStatusBar()->SetStatusText(_("Ready"));


    // min size for the frame itself isn't completely done.
    // see the end up wxAuiManager::Update() for the test
    // code. For now, just hard code a frame minimum size
    SetMinSize(wxSize(400,300));


    log = new wxTextCtrl(this,wxID_ANY, "",
						 wxPoint(0,0), wxSize(150,90),
						 wxNO_BORDER | wxTE_MULTILINE | wxTE_READONLY);

	oredirector = new wxStreamToTextRedirector(log);

    m_mgr.AddPane(log, wxAuiPaneInfo().
                  Name(wxT("log")).Caption(wxT("Log Ausgabe")).
                  Bottom().Layer(1).Position(1).MaximizeButton());


   // create the notebook off-window to avoid flicker
	wxSize client_size = GetClientSize();

	nb = new wxAuiNotebook(this, wxID_ANY,
						   wxPoint(client_size.x, client_size.y),
						   wxSize(430,200),
						   m_notebook_style);

	wxBitmap page_bmp = wxArtProvider::GetBitmap(wxART_NORMAL_FILE, wxART_OTHER, wxSize(16,16));

	m_mgr.AddPane(nb, wxAuiPaneInfo().Name(wxT("content")).
				  CenterPane().PaneBorder(false));

	m_mgr.AddPane(CreateTreeCtrl(), wxAuiPaneInfo().Name(wxT("treectrl")).Caption(wxT("Dateien")).Left());

	CreateSettings();

	CreateToolbar();

	// make some default perspectives

	m_perspective_default = m_mgr.SavePerspective();

	wxString tmp;

	tmp = m_config->Read("Panels", "");
	cout << "Perspective loaded: \"" << tmp << "\"\n";
	m_mgr.LoadPerspective(tmp);

	int x, y, width, height;

	m_config->Read("Left", (long *)&x, (long)100);
	m_config->Read("Top", (long *)&y, (long)100);

	m_config->Read("Width", (long *)&width, (long)800);
	m_config->Read("Height", (long *)&height, (long)600);

	x-=100;
	y-=100;

	SetSize(x, y, width, height);

	m_config->Read("ControlCharacter", (long *)&x, (long)1);
	SetControlCharacter(x);

    m_mgr.Update();

    ActualizeToolbar();
    RefreshOpenFileList();

    m_newFileId=ID_NEW_PROGRAMM;
    m_findreplaceData=NULL;
    m_findreplaceDialog=NULL;
}

MyFrame::~MyFrame()
{
	m_manager->FileHistorySave(*m_config);

	int x, y;
	GetScreenPosition(&x, &y);
	//Es koennen nur posive Werte gespeichert werden, doch wenn das Fenster unter Windows
	//maximiert wird sind die Koordinaten kleiner 0
	x+=100;
	y+=100;
	m_config->Write("Left", (long)x);
	m_config->Write("Top", (long)y);

	GetSize(&x, &y);
	m_config->Write("Width", (long)x);
	m_config->Write("Height", (long)y);

	m_config->Write("ControlCharacter", (m_manager->GetShowControlCharacter()?1:0));

	m_config->Write("Panels", m_mgr.SavePerspective());
    m_mgr.UnInit();
    delete oredirector;
    delete m_config;
	delete m_manager;
}

void MyFrame::InactiveFilesettings()
{
	if(!m_filesettings_active)
		return;

	m_fs_panel->AppendIn(m_filesettings, wxStringProperty(wxT("Keine Datei"),wxT("noFile"),strOPENED));
	m_fs_panel->DisableProperty(wxT("noFile"));
	m_filesettings_active = false;
}

void MyFrame::InactiveProjectsettings()
{
	if(!m_prjsettings_active)
		return;

	m_prjsettings = m_fs_panel->AppendCategory(wxT("Projekt"));
	m_fs_panel->AppendIn(m_prjsettings, wxStringProperty(wxT("Kein Projekt"),wxT("noPrj"),strOPENED));
	m_prjsettings_active = false;
}

void MyFrame::SetActivePrj(Project & prj)
{
	//TODO:SetActivePrj
}

void MyFrame::OnCommentChanged(wxPropertyGridEvent& evt)
{
	cout << "MyFrame::OnCommentChanged()\n";
	if(m_manager->GetCurrentDocument())
	{
		if(m_manager->GetCurrentDocument()->GetDocumentName()!="Programm" && m_manager->GetCurrentDocument()->GetDocumentName()!="Unterprogramm")
		{
			InactiveFilesettings();
			cout << "Aktives Dokument ist weder ein Programm noch ein Unterprogramm!! ->" << __FILE__ << ":" << __LINE__ << endl;
			wxLogError("Aktives Dokument ist weder ein Programm noch ein Unterprogramm!! -> %s:%i\n", __FILE__, __LINE__);
			//wxASSERT(false);
			return;
		}

		if(!m_filesettings_active)
		{
			return;
		}

		wxString tmp = m_fs_panel->GetPropertyValueAsString(wxT("Kommentar"));
		if(tmp.Len() > 20)
		{
			wxLogMessage("Der Kommentar darf maximal 20 Zeichen lang sein. Ihr Kommentar ist %i Zeichen lang. Es werden die ersten 20 Zeichen verwendet.", tmp.Len());
			tmp = tmp.SubString(0, 20);
			m_fs_panel->SetPropertyValue(wxT("Kommentar"), tmp);
		}
		CNCFile * doc =(CNCFile *) m_manager->GetCurrentDocument();
		doc->SetComment(tmp);
	}
}

void MyFrame::RefreshFilesettings()
{
	if(m_manager->GetCurrentDocument())
	{
		if(m_manager->GetCurrentDocument()->GetDocumentName()!="Programm" && m_manager->GetCurrentDocument()->GetDocumentName()!="Unterprogramm")
		{
			InactiveFilesettings();
			cout << "Aktives Dokument ist weder ein Programm noch ein Unterprogramm!! ->" << __FILE__ << ":" << __LINE__ << endl;
			wxLogError("Aktives Dokument ist weder ein Programm noch ein Unterprogramm (%s)!! -> %s:%i\n", m_manager->GetCurrentDocument()->GetDocumentName().c_str(),  __FILE__, __LINE__);
			//wxASSERT(false);
			return;
		}

		if(!m_filesettings_active)
		{
			m_fs_panel->Delete(wxT("noFile"));
			m_fs_panel->AppendIn(m_filesettings, wxStringProperty(wxT("Pfad"),wxT("DateiPfad"),wxT("")));
			m_fs_panel->DisableProperty(wxT("DateiPfad"));
			m_fs_panel->AppendIn(m_filesettings, wxLongStringProperty(wxT("Kommentar"), wxT("Kommentar"), wxT("")));
			m_filesettings_active = true;
		}
		CNCFile * doc =(CNCFile *) m_manager->GetCurrentDocument();
		m_fs_panel->SetPropertyValue(wxT("DateiPfad"), doc->GetFilename());
		m_fs_panel->SetPropertyValue(wxT("Kommentar"), doc->GetComment());
	}
	else
		InactiveFilesettings();
	//TODO:Aenderungen speichern
}

void MyFrame::CreateSettings()
{
	m_filesettings_active = false;
	m_prjsettings_active = false;

	m_fs_panel = new wxPropertyGrid(this, ID_FILESETTINGS, wxDefaultPosition, wxDefaultSize,
		/*wxPG_AUTO_SORT | */wxPG_SPLITTER_AUTO_CENTER | wxPG_DEFAULT_STYLE);

	m_filesettings = m_fs_panel->AppendCategory(wxT("Datei"));
	m_fs_panel->AppendIn(m_filesettings, wxStringProperty(wxT("Keine Datei"),wxT("noFile"),strOPENED));
	m_fs_panel->DisableProperty(wxT("noFile"));

	m_prjsettings = m_fs_panel->AppendCategory(wxT("Projekt"));
	m_fs_panel->AppendIn(m_prjsettings, wxStringProperty(wxT("Kein Projekt"),wxT("noPrj"),strOPENED));
	m_fs_panel->DisableProperty(wxT("noPrj"));

	m_mgr.AddPane(m_fs_panel, wxAuiPaneInfo().Name(wxT("settings")).Caption(wxT("Dateieigenschaften")).Right().MinSize(100, 100));
}


wxAuiDockArt* MyFrame::GetDockArt()
{
    return m_mgr.GetArtProvider();
}

void MyFrame::DoUpdate()
{
    m_mgr.Update();
}

void MyFrame::OnEraseBackground(wxEraseEvent& event)
{
    event.Skip();
}

void MyFrame::OnSize(wxSizeEvent& event)
{
    event.Skip();
}

void MyFrame::OnPaneClose(wxAuiManagerEvent& evt)
{
	/*
    if (evt.pane->name == wxT("test10"))
    {
        int res = wxMessageBox(wxT("Are you sure you want to close/hide this pane?"),
                               wxT("wxAUI"),
                               wxYES_NO,
                               this);
        if (res != wxYES)
            evt.Veto();
    }*/
}


void MyFrame::OnNotebookPageClose(wxAuiNotebookEvent& evt)
{
   	std::cout << "Close Tab\n";
    wxAuiNotebook* ctrl = (wxAuiNotebook*)evt.GetEventObject();
    if (ctrl->GetPage(evt.GetSelection())->IsKindOf(CLASSINFO(MyTextWindow)))
    {
    	CNCFile * doc = ((CNCFile *)((TextEditView *)((MyTextWindow *)ctrl->GetPage(evt.GetSelection()))->view)->doc);
    	std::cout << "Close Tab with Document\n";
    	if(!doc->Close())
    	{
			std::cout << "Event vetoed\n";
            evt.Veto();
    	}
    	else
    	{
    		delete doc;
    	}
    }
    ActualizeToolbar();
    RefreshOpenFileList();
    //FIXME:Wenn Abbrechen gedrckt wird beentet sich das Programm!!!
}

void MyFrame::OnAllowNotebookDnD(wxAuiNotebookEvent& evt)
{
    // allow all noteboko drag and drop events
    evt.Allow();
}

wxPoint MyFrame::GetStartPosition()
{
    static int x = 0;
    x += 20;
    wxPoint pt = ClientToScreen(wxPoint(0,0));
    return wxPoint(pt.x + x, pt.y + x);
}

void MyFrame::OnExit(wxCommandEvent& WXUNUSED(event))
{
    Close(true);
}

void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
	wxAboutDialogInfo info;
	info.AddDeveloper("Andreas Butti");
	info.SetCopyright("(c) 2006 Andreas Butti");
	info.SetDescription("Liest und Schreibt CNi Floppies");
	info.SetIcon(wxIcon(icon_xpm));
	info.SetVersion("0.0.1");
	wxAboutBox(info);
}


wxTreeCtrl* MyFrame::CreateTreeCtrl()
{
    m_tree = new wxTreeCtrl(this, ID_Tree,
                                      wxPoint(0,0), wxSize(160,250),
                                      wxTR_DEFAULT_STYLE | wxNO_BORDER);

	wxColourDatabase * color;
	color = new wxColourDatabase;

    wxImageList* imglist = new wxImageList(16, 16, true, 2);
    for(int i = 0; i < Image::BITMAPS; i++)
		imglist->Add(*Image::Get(i));
    m_tree->AssignImageList(imglist);

    wxTreeItemId root = m_tree->AddRoot(wxT("Dateien"), 0);

	m_floppy = m_tree->AppendItem(root, "Floppy", 1);
	m_prjRoot = m_tree->AppendItem(root, "Projekte", 2, 3);
    m_tree->Expand(m_prjRoot);
	m_openedFiles = m_tree->AppendItem(root, strOPENED_FILES, 8);

/*    wxArrayTreeItemIds items;



    items.Add(tree->AppendItem(root, wxT("Item 1"), 0));
    items.Add(tree->AppendItem(root, wxT("Item 2"), 0));
    items.Add(tree->AppendItem(root, wxT("Item 3"), 0));
    items.Add(tree->AppendItem(root, wxT("Item 4"), 0));
    items.Add(tree->AppendItem(root, wxT("Item 5"), 0));


    int i, count;
    for (i = 0, count = items.Count(); i < count; ++i)
    {
        wxTreeItemId id = items.Item(i);
        tree->AppendItem(id, wxT("Subitem 1"), 1);
        tree->AppendItem(id, wxT("Subitem 2"), 1);
        tree->AppendItem(id, wxT("Subitem 3"), 1);
        tree->AppendItem(id, wxT("Subitem 4"), 1);
        tree->AppendItem(id, wxT("Subitem 5"), 1);
    }

*/
    m_tree->Expand(root);

    return m_tree;
}

#define ACTIVATE_TOOL(tb, ID, activ) \
	if(tb) \
		tb->EnableTool(ID, activ)

void MyFrame::ActualizeToolbar()
{
	int tools=0;
	CNCFile * currentDocument = (CNCFile *)m_manager->GetCurrentDocument();
	if(currentDocument)
		tools=currentDocument->GetActiveToolbars();

	//ACTIVATE_TOOL(tb_file, wxID_SAVE, (tools & CNCFile::ID_TB_SAVE));
	ACTIVATE_TOOL(tb_file, ID_TB_PFD, (tools & CNCFile::ID_TB_SAVE_AS_PDF));
	ACTIVATE_TOOL(tb_edit, wxID_CUT, (tools & CNCFile::ID_TB_CUT));
	ACTIVATE_TOOL(tb_edit, wxID_COPY, (tools & CNCFile::ID_TB_COPY));
	ACTIVATE_TOOL(tb_edit, wxID_PASTE, (tools & CNCFile::ID_TB_PASTE));
	ACTIVATE_TOOL(tb_edit, wxID_FIND, (tools & CNCFile::ID_TB_SEARCH));
	ACTIVATE_TOOL(tb_edit, ID_UNDO, (tools & CNCFile::ID_TB_UNDO));
	ACTIVATE_TOOL(tb_edit, ID_REDO, (tools & CNCFile::ID_TB_REDO));
}

void MyFrame::OnNotebookChanged(wxAuiNotebookEvent& evt)
{
	NotebookChanged();
}

void MyFrame::NotebookChanged()
{
	cout << "Notebook changed\n";
    if (nb->GetPage(nb->GetSelection())->IsKindOf(CLASSINFO(MyTextWindow)))
    {
    	CNCFile * doc = ((CNCFile *)((TextEditView *)((MyTextWindow *)nb->GetPage(nb->GetSelection()))->view)->doc);
    	std::cout << "Notebook changed with Document\n";
		doc->GetDocumentManager()->ActivateView((TextEditView *)((MyTextWindow *)nb->GetPage(nb->GetSelection()))->view, true);
    }

	RefreshFilesettings();

    ActualizeToolbar();
}

void MyFrame::OnDocChanged(wxScintillaEvent & evt)
{
	if(m_manager->GetCurrentDocument())
		m_manager->GetCurrentDocument()->Modify(true);

	ActualizeToolbar();
}

void MyFrame::copy(wxCommandEvent& evt)
{
	CNCFile * currentDocument = (CNCFile *)m_manager->GetCurrentDocument();
	if(currentDocument)
		currentDocument->copy();
}

void MyFrame::paste(wxCommandEvent& evt)
{
	CNCFile * currentDocument = (CNCFile *)m_manager->GetCurrentDocument();
	if(currentDocument)
		currentDocument->paste();
}

void MyFrame::cut(wxCommandEvent& evt)
{
	CNCFile * currentDocument = (CNCFile *)m_manager->GetCurrentDocument();
	if(currentDocument)
		currentDocument->cut();
}

void MyFrame::undo(wxCommandEvent& evt)
{
	CNCFile * currentDocument = (CNCFile *)m_manager->GetCurrentDocument();
	if(currentDocument)
		currentDocument->undo();
}

void MyFrame::redo(wxCommandEvent& evt)
{
	CNCFile * currentDocument = (CNCFile *)m_manager->GetCurrentDocument();
	if(currentDocument)
		currentDocument->redo();
}

void MyFrame::search(wxCommandEvent& evt)
{
	SearchReplace(true, evt.GetId()==ID_FIND_CONTINUE);
}

void MyFrame::replace(wxCommandEvent& evt)
{
	SearchReplace(false);
}

void MyFrame::SearchReplace(bool search, bool quiet)
{

	cout << "SearchReplace(" << search << ", " << quiet << ")\n";

	//TODO:Implementieren
	if(!m_findreplaceData)
		m_findreplaceData = new wxFindReplaceData();
	m_findreplaceData->SetFlags((search?0:wxFR_REPLACEDIALOG));
	wxDELETE(m_findreplaceDialog);

	m_findreplaceDialog = new wxFindReplaceDialog(this, m_findreplaceData, "Suchen");
	m_findreplaceDialog->Show();
}

void MyFrame::OnFindDialog(wxFindDialogEvent & evt)
{
	//TODO:Implementieren
    wxEventType type = evt.GetEventType();

    if ( type == wxEVT_COMMAND_FIND || type == wxEVT_COMMAND_FIND_NEXT )
    {
        wxLogMessage(wxT("Find %s'%s' (flags: %s)"),
                     type == wxEVT_COMMAND_FIND_NEXT ? wxT("next ") : wxT(""),
                     evt.GetFindString().c_str(),
                     "");
//                     DecodeFindDialogEventFlags(evt.GetFlags()).c_str());
    }
    else if ( type == wxEVT_COMMAND_FIND_REPLACE ||
                type == wxEVT_COMMAND_FIND_REPLACE_ALL )
    {
        wxLogMessage(wxT("Replace %s'%s' with '%s' (flags: %s)"),
                     type == wxEVT_COMMAND_FIND_REPLACE_ALL ? _T("all ") : wxT(""),
                     evt.GetFindString().c_str(),
                     evt.GetReplaceString().c_str(),
                     "");
//                     DecodeFindDialogEventFlags(evt.GetFlags()).c_str());
    }
    else if ( type == wxEVT_COMMAND_FIND_CLOSE )
    {
		wxMessageBox("Close");
    }
    else
    {
        wxLogError(wxT("Unknown find dialog event!"));
    }
	/*
	CNCFile * currentDocument = (CNCFile *)m_manager->GetCurrentDocument();
	if(currentDocument)
		currentDocument->search();
	*/

}

void MyFrame::save_as_pdf(wxCommandEvent& evt)
{
	CNCFile * currentDocument = (CNCFile *)m_manager->GetCurrentDocument();
	if(currentDocument)
		currentDocument->save_as_pdf();
}

void MyFrame::SetView(wxCommandEvent& evt)
{
	cout << "SetView called, ID = " << evt.GetId() << ", Checked = " << evt.IsChecked() << "\n";
	switch(evt.GetId())
	{
		case ID_PSHOW_PRJ: m_mgr.GetPane(wxT("treectrl")).Show();
			break;
		case ID_PSHOW_LOG: m_mgr.GetPane(wxT("log")).Show();
			break;
		case ID_PSHOW_SETTINGS: m_mgr.GetPane(wxT("settings")).Show();
			break;
		case ID_SHOWTB_FILE: m_mgr.GetPane(wxT("tb_file")).Show();
			break;
		case ID_SHOWTB_EDIT: m_mgr.GetPane(wxT("tb_edit")).Show();
			break;
		case ID_SHOWTB_FLOPPY: m_mgr.GetPane(wxT("tb_floppy")).Show();
			break;
		case ID_SHOWTB_VIEW: m_mgr.GetPane(wxT("tb_view")).Show();
			break;
		default:
			return;
	}
	m_mgr.Update();

}

void MyFrame::LoadDefaultView(wxCommandEvent& evt)
{
	m_mgr.LoadPerspective(m_perspective_default, true);
}

void MyFrame::RefreshOpenFileList()
{
	cout << "RefreshOpenFileList()\n";

	m_tree->DeleteChildren(m_openedFiles);

	wxList & docs = m_manager->GetDocuments();
	wxDocument * doc;
	wxString tmp;

	m_openedFilesList.clear();

	TextEditView *view;
	wxString tmpstr;
	int treeid;

	for(unsigned int i=0; i < docs.GetCount(); i++)
	{
		doc = (wxDocument *)docs.Item(i)->GetData();
		doc->GetPrintableName(tmp);
		if(doc->GetDocumentName()=="Programm")
		{
			view = (TextEditView *)doc->GetFirstView();
			treeid = (int)m_tree->AppendItem(m_openedFiles, tmp, 6).m_pItem;
			m_openedFilesList[treeid]=view->textsw;
			doc->GetPrintableName(tmpstr);
			cout << "TreeID = " << treeid << "; " << tmpstr << "\n";
		}
		else if(doc->GetDocumentName()=="Unterprogramm")
		{
			view = (TextEditView *)doc->GetFirstView();
			treeid = (int)m_tree->AppendItem(m_openedFiles, tmp, 7).m_pItem;
			m_openedFilesList[treeid]=view->textsw;;
			doc->GetPrintableName(tmpstr);
			cout << "TreeID = " << treeid << "; " << tmpstr << "\n";
		}
	}
	m_tree->Expand(m_openedFiles);
}

void MyFrame::TreeBeginDrag(wxTreeEvent & evt)
{
	evt.Allow();
	//GetItem

	//TODO:Drag implementieren
}

void MyFrame::NewProgramm(wxCommandEvent& evt)
{
	m_manager->NewDocument(wxString("*.prog"), wxDOC_SILENT);
	SetLastNewDocument(ID_NEW_PROGRAMM);
}

void MyFrame::NewUProgramm(wxCommandEvent& evt)
{
	m_manager->NewDocument(wxString("*.uprog"), wxDOC_SILENT);
	SetLastNewDocument(ID_NEW_UPROGRAMM);
}

void MyFrame::NewProject(wxCommandEvent& evt)
{
	m_manager->NewDocument(wxString("*.prj"), wxDOC_SILENT);
	SetLastNewDocument(ID_NEW_PROJECT);
}

void MyFrame::SetLastNewDocument(int id)
{
	m_newFileId = id;

	m_new->SetLabel(ID_NEW_PROGRAMM, wxString(wxT("Programm")) + ((id == ID_NEW_PROGRAMM) ? wxString(wxT("\tCtrl-N")) : wxString(wxT(""))));
	m_new->SetLabel(ID_NEW_UPROGRAMM, wxString(wxT("Unterprogramm")) + ((id == ID_NEW_UPROGRAMM) ? wxString(wxT("\tCtrl-N")) : wxString(wxT(""))));
	m_new->SetLabel(ID_NEW_PROJECT, wxString(wxT("Projekt")) + ((id == ID_NEW_PROJECT) ? wxString(wxT("\tCtrl-N")) : wxString(wxT(""))));

	RefreshOpenFileList();
}

void MyFrame::NewToolbar(wxCommandEvent& evt)
{
	switch(m_newFileId)
	{
		case ID_NEW_PROGRAMM:
			m_manager->NewDocument(wxString("*.prog"), wxDOC_SILENT);
			break;
		case ID_NEW_UPROGRAMM:
			m_manager->NewDocument(wxString("*.uprog"), wxDOC_SILENT);
			break;
		case ID_NEW_PROJECT:
			m_manager->NewDocument(wxString("*.prj"), wxDOC_SILENT);
			break;
	}
}

void MyFrame::FillNewPulldown(wxCommandEvent& event)
{
	wxMenu *menu = new wxMenu();
	wxMenuItem *tmp;

	tmp = new wxMenuItem(menu, ID_NEW_PROGRAMM, wxString(wxT("Programm")) , wxT(""), wxITEM_NORMAL);
	tmp->SetBitmap(*Image::Get(6));
	menu->Append(tmp);
	tmp = new wxMenuItem(menu, ID_NEW_UPROGRAMM, wxString(wxT("Unterprogramm")) , wxT(""), wxITEM_NORMAL);
	tmp->SetBitmap(*Image::Get(7));
	menu->Append(tmp);
	tmp = new wxMenuItem(menu, ID_NEW_PROJECT, wxString(wxT("Projekt")) , wxT(""), wxITEM_NORMAL);
	tmp->SetBitmap(*Image::Get(5));
	menu->Append(tmp);

//	PopupMenu(menu, x, y);
	PopupMenu(menu);
}

void MyFrame::TreeItemActivated(wxTreeEvent & evt)
{
	cout << "TreeItemActivated; ID = " << (int)evt.GetItem().m_pItem << "\n";

	if(m_tree->GetItemParent(evt.GetItem())==m_openedFiles)
	{
		//Geoefnetet Datei aus Der liste Doppelgeklickt
		int id = nb->GetPageIndex(m_openedFilesList[(int)evt.GetItem().m_pItem]);
		if(id == wxNOT_FOUND)
			return;
		nb->SetSelection(id);
	}
	else
	{
		//Alle Projekte durchloopen
		wxList & docs = m_manager->GetDocuments();
		wxDocument * doc;
		for(unsigned int i=0; i < docs.GetCount(); i++)
		{
			doc = (wxDocument *)docs.Item(i)->GetData();
			if(doc->GetDocumentName()=="Projekt")
			{
				if(((ProjectView *)doc->GetFirstView())->DblClick(evt.GetItem()))
					break; //Wenn es ein Item dieses Projektes ist nicht weiter suchen
			}
		}
	}

}

void MyFrame::toggleControlCharacter(wxCommandEvent& evt)
{
	SetControlCharacter((m_manager->GetShowControlCharacter()==false));
}

void MyFrame::SetControlCharacter(bool cc)
{
	cout << "SetControlCharacter(" << (cc?"true":"false") << ")\n";
	m_manager->SetShowControlCharacter(cc);
	tb_view->ToggleTool(ID_TB_VIEW_CC, cc);
	GetMenuBar()->GetMenu(4)->Check(ID_VIEW_CC, cc);
}

void MyFrame::ActivateNewstFile()
{
	cout << "ActivateNewstFile()\n";
	//Neusten Tab whlen
	nb->SetSelection(nb->GetPageCount()-1);
	NotebookChanged();
}

void MyFrame::SelectAll(wxCommandEvent& evt)
{
	CNCFile * currentDocument = (CNCFile *)m_manager->GetCurrentDocument();
	if(currentDocument)
		currentDocument->SelectAll();

}














