#ifndef MYFRAME_H
#define MYFRAME_H
#include "MyDocManager.h"
#include <wx/propgrid/propgrid.h>
#include "Project.h"
#include "wx/wxscintilla.h"
#include <wx/fdrepdlg.h>

class wxSizeReportCtrl;
class CNCFile;

WX_DECLARE_HASH_MAP(int, wxWindow*, wxIntegerHash, wxIntegerEqual, PwxWindowHashMap);

class MyFrame : public wxDocParentFrame
{
public:
    enum
    {
        ID_TextEditWindow = wxID_HIGHEST+1,
		ID_TB_PFD,
		ID_TB_READDISK,
		ID_TB_WRITEDISK,
		ID_TB_Settings,
		ID_TB_VIEW_CC,
		ID_SAVE_PRJ,
		ID_PRJ_SAVE_AS,
		ID_PRJ_CLOSE,
		ID_FIND_CONTINUE,
		ID_READ_FLOPPY,
		ID_SAVE_FLOPPY_AS_PRJ,
		ID_CREATE_IMG,
		ID_WRITE_IMG,
		ID_FORMAT,
		ID_PRJ_SAVE_FLOPPY,
		ID_PRJ_ADD,
		ID_PRJ_DEL,
		ID_PRJ_RENAME,
		ID_VIEW_RESET,
		ID_SHOWTB_FILE,
		ID_SHOWTB_EDIT,
		ID_SHOWTB_FLOPPY,
		ID_SHOWTB_VIEW,
		ID_PSHOW_PRJ,
		ID_PSHOW_LOG,
		ID_PSHOW_SETTINGS,
		ID_Tree,
		ID_NEW_PROGRAMM,
		ID_NEW_UPROGRAMM,
		ID_NEW_PROJECT,
		ID_NEW_TOOLBAR,
		ID_NEW_TOOLBAR_DROPDOWN,
		ID_VIEW_CC,
		ID_REDO,
		ID_UNDO,
		//Settings
		ID_FILESETTINGS,



        ID_FirstPerspective
    };

    MyFrame(MyDocManager *manager, wxFrame *frame, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size,
        const long type);

    ~MyFrame();

    wxAuiDockArt* GetDockArt();
    void DoUpdate();
    void ActualizeToolbar();
	CNCFile * CurrentDocument();
	wxAuiNotebook * nb;
	void OnDocChanged(wxScintillaEvent & evt);

	//Toolbarfunktionen alle klein geschrieben
	void copy(wxCommandEvent& evt);
	void paste(wxCommandEvent& evt);
	void cut(wxCommandEvent& evt);
	void undo(wxCommandEvent& evt);
	void redo(wxCommandEvent& evt);
	void search(wxCommandEvent& evt);
	void replace(wxCommandEvent& evt);
	void settings(wxCommandEvent& evt);
	void save_as_pdf(wxCommandEvent& evt);
	void toggleControlCharacter(wxCommandEvent& evt);

	void SearchReplace(bool search, bool quiet = false);

	void SelectAll(wxCommandEvent& evt);

	void SetView(wxCommandEvent& evt);

    void RefreshOpenFileList();

    wxTreeItemId m_floppy;
    wxTreeItemId m_prjRoot;
    wxTreeCtrl* m_tree;

	void NewProgramm(wxCommandEvent& evt);
	void NewUProgramm(wxCommandEvent& evt);
	void NewProject(wxCommandEvent& evt);

    void TreeBeginDrag(wxTreeEvent & evt);

	void FillNewPulldown(wxCommandEvent& event);
	void NewToolbar(wxCommandEvent& evt);

	void TreeItemActivated(wxTreeEvent & evt);

	void ActivateNewstFile();
	void NotebookChanged();

private:

	int m_newFileId;

    wxTreeCtrl* CreateTreeCtrl();
    wxPoint GetStartPosition();

    wxString GetIntroText();

    void OnEraseBackground(wxEraseEvent& evt);
    void OnSize(wxSizeEvent& evt);

    void OnNotebookPageClose(wxAuiNotebookEvent& evt);
    void OnExit(wxCommandEvent& evt);
    void OnAbout(wxCommandEvent& evt);

    void OnAllowNotebookDnD(wxAuiNotebookEvent& evt);
    void OnNotebookChanged(wxAuiNotebookEvent& evt);
	void LoadDefaultView(wxCommandEvent& evt);

    void OnPaneClose(wxAuiManagerEvent& evt);
    void CreateToolbar();
    void CreateMenuBar();
    void SetLastNewDocument(int id);

	void SetControlCharacter(bool cc);


	//Settings
	void OnCommentChanged(wxPropertyGridEvent& evt);

	wxTextCtrl * log;
	wxStreamToTextRedirector * oredirector;
    wxAuiManager m_mgr;
    wxMenu* m_perspectives_menu;
	wxMenu* m_historyMenu;
	wxMenu* m_new;

	wxConfig * m_config;

    long m_notebook_style;
    long m_notebook_theme;
	wxString m_perspective_default;
    DECLARE_EVENT_TABLE()

	MyDocManager *m_manager;
	wxToolBar * tb_file;

	wxToolBar * tb_edit;
	wxToolBar * tb_floppy;
	wxToolBar * tb_view;

    wxTreeItemId m_openedFiles;

    PwxWindowHashMap m_openedFilesList;

	//Filessettings
	bool m_filesettings_active, m_prjsettings_active;
	wxPGId m_filesettings, m_prjsettings;

	void CreateSettings();
	void InactiveProjectsettings();
	void InactiveFilesettings();
	wxPropertyGrid * m_fs_panel;
	void RefreshFilesettings();
	void SetActivePrj(Project & prj);

	wxFindReplaceData * m_findreplaceData;
	wxFindReplaceDialog * m_findreplaceDialog;
	void OnFindDialog(wxFindDialogEvent & evt);

};


#endif //MYFRAME_H
